"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .taxidformat import TaxIDFormat
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


CustomerCreateMetadataTypedDict = TypeAliasType(
    "CustomerCreateMetadataTypedDict", Union[str, int, bool]
)


CustomerCreateMetadata = TypeAliasType("CustomerCreateMetadata", Union[str, int, bool])


CustomerCreateTaxIDTypedDict = TypeAliasType(
    "CustomerCreateTaxIDTypedDict", Union[str, TaxIDFormat]
)


CustomerCreateTaxID = TypeAliasType("CustomerCreateTaxID", Union[str, TaxIDFormat])


class CustomerCreateTypedDict(TypedDict):
    email: str
    r"""The email address of the customer. This must be unique within the organization."""
    metadata: NotRequired[Dict[str, CustomerCreateMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    external_id: NotRequired[Nullable[str]]
    r"""The ID of the customer in your system. This must be unique within the organization. Once set, it can't be updated."""
    name: NotRequired[Nullable[str]]
    r"""The name of the customer."""
    billing_address: NotRequired[Nullable[AddressTypedDict]]
    tax_id: NotRequired[Nullable[List[Nullable[CustomerCreateTaxIDTypedDict]]]]
    organization_id: NotRequired[Nullable[str]]
    r"""The ID of the organization owning the customer. **Required unless you use an organization token.**"""


class CustomerCreate(BaseModel):
    email: str
    r"""The email address of the customer. This must be unique within the organization."""

    metadata: Optional[Dict[str, CustomerCreateMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    external_id: OptionalNullable[str] = UNSET
    r"""The ID of the customer in your system. This must be unique within the organization. Once set, it can't be updated."""

    name: OptionalNullable[str] = UNSET
    r"""The name of the customer."""

    billing_address: OptionalNullable[Address] = UNSET

    tax_id: OptionalNullable[List[Nullable[CustomerCreateTaxID]]] = UNSET

    organization_id: OptionalNullable[str] = UNSET
    r"""The ID of the organization owning the customer. **Required unless you use an organization token.**"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "metadata",
            "external_id",
            "name",
            "billing_address",
            "tax_id",
            "organization_id",
        ]
        nullable_fields = [
            "external_id",
            "name",
            "billing_address",
            "tax_id",
            "organization_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class CustomerBenefitGrantDownloadablesUpdateTypedDict(TypedDict):
    benefit_type: Literal["downloadables"]


class CustomerBenefitGrantDownloadablesUpdate(BaseModel):
    BENEFIT_TYPE: Annotated[
        Annotated[
            Literal["downloadables"], AfterValidator(validate_const("downloadables"))
        ],
        pydantic.Field(alias="benefit_type"),
    ] = "downloadables"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .organizationsocialplatforms import OrganizationSocialPlatforms
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class OrganizationSocialLinkTypedDict(TypedDict):
    platform: OrganizationSocialPlatforms
    url: str
    r"""The URL to the organization profile"""


class OrganizationSocialLink(BaseModel):
    platform: OrganizationSocialPlatforms

    url: str
    r"""The URL to the organization profile"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customercancellationreason import CustomerCancellationReason
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, NotRequired, TypedDict


class SubscriptionCancelTypedDict(TypedDict):
    cancel_at_period_end: NotRequired[Nullable[bool]]
    r"""Cancel an active subscription once the current period ends.

    Or uncancel a subscription currently set to be revoked at period end.
    """
    revoke: Nullable[Literal[True]]
    r"""Cancel and revoke an active subscription immediately"""
    customer_cancellation_reason: NotRequired[Nullable[CustomerCancellationReason]]
    r"""Customer reason for cancellation.

    Helpful to monitor reasons behind churn for future improvements.

    Only set this in case your own service is requesting the reason from the
    customer. Or you know based on direct conversations, i.e support, with
    the customer.

    * `too_expensive`: Too expensive for the customer.
    * `missing_features`: Customer is missing certain features.
    * `switched_service`: Customer switched to another service.
    * `unused`: Customer is not using it enough.
    * `customer_service`: Customer is not satisfied with the customer service.
    * `low_quality`: Customer is unhappy with the quality.
    * `too_complex`: Customer considers the service too complicated.
    * `other`: Other reason(s).
    """
    customer_cancellation_comment: NotRequired[Nullable[str]]
    r"""Customer feedback and why they decided to cancel.

    **IMPORTANT:**
    Do not use this to store internal notes! It's intended to be input
    from the customer and is therefore also available in their Polar
    purchases library.

    Only set this in case your own service is requesting the reason from the
    customer. Or you copy a message directly from a customer
    conversation, i.e support.
    """


class SubscriptionCancel(BaseModel):
    cancel_at_period_end: OptionalNullable[bool] = UNSET
    r"""Cancel an active subscription once the current period ends.

    Or uncancel a subscription currently set to be revoked at period end.
    """

    REVOKE: Annotated[
        Annotated[
            OptionalNullable[Literal[True]], AfterValidator(validate_const(True))
        ],
        pydantic.Field(alias="revoke"),
    ] = True
    r"""Cancel and revoke an active subscription immediately"""

    customer_cancellation_reason: OptionalNullable[CustomerCancellationReason] = UNSET
    r"""Customer reason for cancellation.

    Helpful to monitor reasons behind churn for future improvements.

    Only set this in case your own service is requesting the reason from the
    customer. Or you know based on direct conversations, i.e support, with
    the customer.

    * `too_expensive`: Too expensive for the customer.
    * `missing_features`: Customer is missing certain features.
    * `switched_service`: Customer switched to another service.
    * `unused`: Customer is not using it enough.
    * `customer_service`: Customer is not satisfied with the customer service.
    * `low_quality`: Customer is unhappy with the quality.
    * `too_complex`: Customer considers the service too complicated.
    * `other`: Other reason(s).
    """

    customer_cancellation_comment: OptionalNullable[str] = UNSET
    r"""Customer feedback and why they decided to cancel.

    **IMPORTANT:**
    Do not use this to store internal notes! It's intended to be input
    from the customer and is therefore also available in their Polar
    purchases library.

    Only set this in case your own service is requesting the reason from the
    customer. Or you copy a message directly from a customer
    conversation, i.e support.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "cancel_at_period_end",
            "revoke",
            "customer_cancellation_reason",
            "customer_cancellation_comment",
        ]
        nullable_fields = [
            "cancel_at_period_end",
            "revoke",
            "customer_cancellation_reason",
            "customer_cancellation_comment",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkoutlinkupdate import CheckoutLinkUpdate, CheckoutLinkUpdateTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class CheckoutLinksUpdateRequestTypedDict(TypedDict):
    id: str
    r"""The checkout link ID."""
    checkout_link_update: CheckoutLinkUpdateTypedDict


class CheckoutLinksUpdateRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The checkout link ID."""

    checkout_link_update: Annotated[
        CheckoutLinkUpdate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PledgeType(str, Enum):
    PAY_UPFRONT = "pay_upfront"
    PAY_ON_COMPLETION = "pay_on_completion"
    PAY_DIRECTLY = "pay_directly"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .countaggregation import CountAggregation, CountAggregationTypedDict
from .filter_ import Filter, FilterTypedDict
from .propertyaggregation import PropertyAggregation, PropertyAggregationTypedDict
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import get_discriminator
import pydantic
from pydantic import Discriminator, Tag, model_serializer
from typing import Dict, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


MeterCreateMetadataTypedDict = TypeAliasType(
    "MeterCreateMetadataTypedDict", Union[str, int, bool]
)


MeterCreateMetadata = TypeAliasType("MeterCreateMetadata", Union[str, int, bool])


MeterCreateAggregationTypedDict = TypeAliasType(
    "MeterCreateAggregationTypedDict",
    Union[CountAggregationTypedDict, PropertyAggregationTypedDict],
)
r"""The aggregation to apply on the filtered events to calculate the meter."""


MeterCreateAggregation = Annotated[
    Union[
        Annotated[PropertyAggregation, Tag("avg")],
        Annotated[CountAggregation, Tag("count")],
        Annotated[PropertyAggregation, Tag("max")],
        Annotated[PropertyAggregation, Tag("min")],
        Annotated[PropertyAggregation, Tag("sum")],
    ],
    Discriminator(lambda m: get_discriminator(m, "func", "func")),
]
r"""The aggregation to apply on the filtered events to calculate the meter."""


class MeterCreateTypedDict(TypedDict):
    name: str
    r"""The name of the meter. Will be shown on customer's invoices and usage."""
    filter_: FilterTypedDict
    aggregation: MeterCreateAggregationTypedDict
    r"""The aggregation to apply on the filtered events to calculate the meter."""
    metadata: NotRequired[Dict[str, MeterCreateMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    organization_id: NotRequired[Nullable[str]]
    r"""The ID of the organization owning the meter. **Required unless you use an organization token.**"""


class MeterCreate(BaseModel):
    name: str
    r"""The name of the meter. Will be shown on customer's invoices and usage."""

    filter_: Annotated[Filter, pydantic.Field(alias="filter")]

    aggregation: MeterCreateAggregation
    r"""The aggregation to apply on the filtered events to calculate the meter."""

    metadata: Optional[Dict[str, MeterCreateMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    organization_id: OptionalNullable[str] = UNSET
    r"""The ID of the organization owning the meter. **Required unless you use an organization token.**"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["metadata", "organization_id"]
        nullable_fields = ["organization_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SubscriptionProrationBehavior(str, Enum):
    INVOICE = "invoice"
    PRORATE = "prorate"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Dict, List
from typing_extensions import TypedDict


class BenefitDownloadablesPropertiesTypedDict(TypedDict):
    archived: Dict[str, bool]
    files: List[str]


class BenefitDownloadablesProperties(BaseModel):
    archived: Dict[str, bool]

    files: List[str]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .listresource_order_ import ListResourceOrder, ListResourceOrderTypedDict
from .ordersortproperty import OrderSortProperty
from .productbillingtype import ProductBillingType
from .productpricetype import ProductPriceType
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import Callable, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


OrdersListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "OrdersListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


OrdersListQueryParamOrganizationIDFilter = TypeAliasType(
    "OrdersListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


OrdersListQueryParamProductIDFilterTypedDict = TypeAliasType(
    "OrdersListQueryParamProductIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by product ID."""


OrdersListQueryParamProductIDFilter = TypeAliasType(
    "OrdersListQueryParamProductIDFilter", Union[str, List[str]]
)
r"""Filter by product ID."""


ProductBillingTypeFilterTypedDict = TypeAliasType(
    "ProductBillingTypeFilterTypedDict",
    Union[ProductBillingType, List[ProductBillingType]],
)
r"""Filter by product billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""


ProductBillingTypeFilter = TypeAliasType(
    "ProductBillingTypeFilter", Union[ProductBillingType, List[ProductBillingType]]
)
r"""Filter by product billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""


ProductPriceTypeFilterTypedDict = TypeAliasType(
    "ProductPriceTypeFilterTypedDict", Union[ProductPriceType, List[ProductPriceType]]
)


ProductPriceTypeFilter = TypeAliasType(
    "ProductPriceTypeFilter", Union[ProductPriceType, List[ProductPriceType]]
)


QueryParamDiscountIDFilterTypedDict = TypeAliasType(
    "QueryParamDiscountIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by discount ID."""


QueryParamDiscountIDFilter = TypeAliasType(
    "QueryParamDiscountIDFilter", Union[str, List[str]]
)
r"""Filter by discount ID."""


OrdersListQueryParamCustomerIDFilterTypedDict = TypeAliasType(
    "OrdersListQueryParamCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by customer ID."""


OrdersListQueryParamCustomerIDFilter = TypeAliasType(
    "OrdersListQueryParamCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by customer ID."""


CheckoutIDFilterTypedDict = TypeAliasType(
    "CheckoutIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by checkout ID."""


CheckoutIDFilter = TypeAliasType("CheckoutIDFilter", Union[str, List[str]])
r"""Filter by checkout ID."""


class OrdersListRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[OrdersListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    product_id: NotRequired[Nullable[OrdersListQueryParamProductIDFilterTypedDict]]
    r"""Filter by product ID."""
    product_billing_type: NotRequired[Nullable[ProductBillingTypeFilterTypedDict]]
    r"""Filter by product billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""
    product_price_type: NotRequired[Nullable[ProductPriceTypeFilterTypedDict]]
    discount_id: NotRequired[Nullable[QueryParamDiscountIDFilterTypedDict]]
    r"""Filter by discount ID."""
    customer_id: NotRequired[Nullable[OrdersListQueryParamCustomerIDFilterTypedDict]]
    r"""Filter by customer ID."""
    checkout_id: NotRequired[Nullable[CheckoutIDFilterTypedDict]]
    r"""Filter by checkout ID."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[OrderSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""


class OrdersListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[OrdersListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    product_id: Annotated[
        OptionalNullable[OrdersListQueryParamProductIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product ID."""

    product_billing_type: Annotated[
        OptionalNullable[ProductBillingTypeFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""

    product_price_type: Annotated[
        OptionalNullable[ProductPriceTypeFilter],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    discount_id: Annotated[
        OptionalNullable[QueryParamDiscountIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by discount ID."""

    customer_id: Annotated[
        OptionalNullable[OrdersListQueryParamCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer ID."""

    checkout_id: Annotated[
        OptionalNullable[CheckoutIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by checkout ID."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[OrderSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "organization_id",
            "product_id",
            "product_billing_type",
            "product_price_type",
            "discount_id",
            "customer_id",
            "checkout_id",
            "page",
            "limit",
            "sorting",
        ]
        nullable_fields = [
            "organization_id",
            "product_id",
            "product_billing_type",
            "product_price_type",
            "discount_id",
            "customer_id",
            "checkout_id",
            "sorting",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class OrdersListResponseTypedDict(TypedDict):
    result: ListResourceOrderTypedDict


class OrdersListResponse(BaseModel):
    next: Callable[[], Optional[OrdersListResponse]]

    result: ListResourceOrder



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


CheckoutLinkCreateProductsMetadataTypedDict = TypeAliasType(
    "CheckoutLinkCreateProductsMetadataTypedDict", Union[str, int, bool]
)


CheckoutLinkCreateProductsMetadata = TypeAliasType(
    "CheckoutLinkCreateProductsMetadata", Union[str, int, bool]
)


class CheckoutLinkCreateProductsTypedDict(TypedDict):
    r"""Schema to create a new checkout link."""

    products: List[str]
    r"""List of products that will be available to select at checkout."""
    metadata: NotRequired[Dict[str, CheckoutLinkCreateProductsMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    payment_processor: Literal["stripe"]
    r"""Payment processor to use. Currently only Stripe is supported."""
    label: NotRequired[Nullable[str]]
    r"""Optional label to distinguish links internally"""
    allow_discount_codes: NotRequired[bool]
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""
    discount_id: NotRequired[Nullable[str]]
    r"""ID of the discount to apply to the checkout. If the discount is not applicable anymore when opening the checkout link, it'll be ignored."""
    success_url: NotRequired[Nullable[str]]
    r"""URL where the customer will be redirected after a successful payment.You can add the `checkout_id={CHECKOUT_ID}` query parameter to retrieve the checkout session id."""


class CheckoutLinkCreateProducts(BaseModel):
    r"""Schema to create a new checkout link."""

    products: List[str]
    r"""List of products that will be available to select at checkout."""

    metadata: Optional[Dict[str, CheckoutLinkCreateProductsMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    PAYMENT_PROCESSOR: Annotated[
        Annotated[Literal["stripe"], AfterValidator(validate_const("stripe"))],
        pydantic.Field(alias="payment_processor"),
    ] = "stripe"
    r"""Payment processor to use. Currently only Stripe is supported."""

    label: OptionalNullable[str] = UNSET
    r"""Optional label to distinguish links internally"""

    allow_discount_codes: Optional[bool] = True
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""

    discount_id: OptionalNullable[str] = UNSET
    r"""ID of the discount to apply to the checkout. If the discount is not applicable anymore when opening the checkout link, it'll be ignored."""

    success_url: OptionalNullable[str] = UNSET
    r"""URL where the customer will be redirected after a successful payment.You can add the `checkout_id={CHECKOUT_ID}` query parameter to retrieve the checkout session id."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "metadata",
            "label",
            "allow_discount_codes",
            "discount_id",
            "success_url",
        ]
        nullable_fields = ["label", "discount_id", "success_url"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .taxidformat import TaxIDFormat
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Union
from typing_extensions import TypeAliasType, TypedDict


CustomerMetadata1TypedDict = TypeAliasType(
    "CustomerMetadata1TypedDict", Union[str, int, bool]
)


CustomerMetadata1 = TypeAliasType("CustomerMetadata1", Union[str, int, bool])


CustomerTaxIDTypedDict = TypeAliasType(
    "CustomerTaxIDTypedDict", Union[str, TaxIDFormat]
)


CustomerTaxID = TypeAliasType("CustomerTaxID", Union[str, TaxIDFormat])


class CustomerTypedDict(TypedDict):
    r"""A customer in an organization."""

    id: str
    r"""The ID of the customer."""
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    metadata: Dict[str, CustomerMetadata1TypedDict]
    external_id: Nullable[str]
    r"""The ID of the customer in your system. This must be unique within the organization. Once set, it can't be updated."""
    email: str
    r"""The email address of the customer. This must be unique within the organization."""
    email_verified: bool
    r"""Whether the customer email address is verified. The address is automatically verified when the customer accesses the customer portal using their email address."""
    name: Nullable[str]
    r"""The name of the customer."""
    billing_address: Nullable[AddressTypedDict]
    tax_id: Nullable[List[Nullable[CustomerTaxIDTypedDict]]]
    organization_id: str
    r"""The ID of the organization owning the customer."""
    deleted_at: Nullable[datetime]
    r"""Timestamp for when the customer was soft deleted."""
    avatar_url: str


class Customer(BaseModel):
    r"""A customer in an organization."""

    id: str
    r"""The ID of the customer."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    metadata: Dict[str, CustomerMetadata1]

    external_id: Nullable[str]
    r"""The ID of the customer in your system. This must be unique within the organization. Once set, it can't be updated."""

    email: str
    r"""The email address of the customer. This must be unique within the organization."""

    email_verified: bool
    r"""Whether the customer email address is verified. The address is automatically verified when the customer accesses the customer portal using their email address."""

    name: Nullable[str]
    r"""The name of the customer."""

    billing_address: Nullable[Address]

    tax_id: Nullable[List[Nullable[CustomerTaxID]]]

    organization_id: str
    r"""The ID of the organization owning the customer."""

    deleted_at: Nullable[datetime]
    r"""Timestamp for when the customer was soft deleted."""

    avatar_url: str

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "modified_at",
            "external_id",
            "name",
            "billing_address",
            "tax_id",
            "deleted_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .listresource_union_paymentmethodcard_paymentmethodgeneric_ import (
    ListResourceUnionPaymentMethodCardPaymentMethodGeneric,
    ListResourceUnionPaymentMethodCardPaymentMethodGenericTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, QueryParamMetadata, SecurityMetadata
from typing import Callable, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CustomerPortalCustomersGetPaymentMethodsSecurityTypedDict(TypedDict):
    customer_session: str


class CustomerPortalCustomersGetPaymentMethodsSecurity(BaseModel):
    customer_session: Annotated[
        str,
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ]


class CustomerPortalCustomersGetPaymentMethodsRequestTypedDict(TypedDict):
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""


class CustomerPortalCustomersGetPaymentMethodsRequest(BaseModel):
    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""


class CustomerPortalCustomersGetPaymentMethodsResponseTypedDict(TypedDict):
    result: ListResourceUnionPaymentMethodCardPaymentMethodGenericTypedDict


class CustomerPortalCustomersGetPaymentMethodsResponse(BaseModel):
    next: Callable[[], Optional[CustomerPortalCustomersGetPaymentMethodsResponse]]

    result: ListResourceUnionPaymentMethodCardPaymentMethodGeneric



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .meterupdate import MeterUpdate, MeterUpdateTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class MetersUpdateRequestTypedDict(TypedDict):
    id: str
    r"""The meter ID."""
    meter_update: MeterUpdateTypedDict


class MetersUpdateRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The meter ID."""

    meter_update: Annotated[
        MeterUpdate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .timeinterval import TimeInterval
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import List, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


MetersQuantitiesQueryParamCustomerIDFilterTypedDict = TypeAliasType(
    "MetersQuantitiesQueryParamCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by customer ID."""


MetersQuantitiesQueryParamCustomerIDFilter = TypeAliasType(
    "MetersQuantitiesQueryParamCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by customer ID."""


QueryParamExternalCustomerIDFilterTypedDict = TypeAliasType(
    "QueryParamExternalCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by external customer ID."""


QueryParamExternalCustomerIDFilter = TypeAliasType(
    "QueryParamExternalCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by external customer ID."""


class MetersQuantitiesRequestTypedDict(TypedDict):
    id: str
    r"""The meter ID."""
    start_timestamp: datetime
    r"""Start timestamp."""
    end_timestamp: datetime
    r"""End timestamp."""
    interval: TimeInterval
    r"""Interval between two timestamps."""
    customer_id: NotRequired[
        Nullable[MetersQuantitiesQueryParamCustomerIDFilterTypedDict]
    ]
    r"""Filter by customer ID."""
    external_customer_id: NotRequired[
        Nullable[QueryParamExternalCustomerIDFilterTypedDict]
    ]
    r"""Filter by external customer ID."""


class MetersQuantitiesRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The meter ID."""

    start_timestamp: Annotated[
        datetime, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""Start timestamp."""

    end_timestamp: Annotated[
        datetime, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""End timestamp."""

    interval: Annotated[
        TimeInterval,
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ]
    r"""Interval between two timestamps."""

    customer_id: Annotated[
        OptionalNullable[MetersQuantitiesQueryParamCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer ID."""

    external_customer_id: Annotated[
        OptionalNullable[QueryParamExternalCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by external customer ID."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["customer_id", "external_customer_id"]
        nullable_fields = ["customer_id", "external_customer_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class RefundReason(str, Enum):
    DUPLICATE = "duplicate"
    FRAUDULENT = "fraudulent"
    CUSTOMER_REQUEST = "customer_request"
    SERVICE_DISRUPTION = "service_disruption"
    SATISFACTION_GUARANTEE = "satisfaction_guarantee"
    OTHER = "other"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customer import Customer, CustomerTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class WebhookCustomerUpdatedPayloadTypedDict(TypedDict):
    r"""Sent when a customer is updated.

    This event is fired when the customer details are updated.

    If you want to be notified when a customer subscription or benefit state changes, you should listen to the `customer_state_changed` event.

    **Discord & Slack support:** Basic
    """

    data: CustomerTypedDict
    r"""A customer in an organization."""
    type: Literal["customer.updated"]


class WebhookCustomerUpdatedPayload(BaseModel):
    r"""Sent when a customer is updated.

    This event is fired when the customer details are updated.

    If you want to be notified when a customer subscription or benefit state changes, you should listen to the `customer_state_changed` event.

    **Discord & Slack support:** Basic
    """

    data: Customer
    r"""A customer in an organization."""

    TYPE: Annotated[
        Annotated[
            Literal["customer.updated"],
            AfterValidator(validate_const("customer.updated")),
        ],
        pydantic.Field(alias="type"),
    ] = "customer.updated"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .paymentmethodcarddata import PaymentMethodCardData, PaymentMethodCardDataTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class PaymentMethodCardTypedDict(TypedDict):
    id: str
    created_at: datetime
    default: bool
    card: PaymentMethodCardDataTypedDict
    type: Literal["card"]


class PaymentMethodCard(BaseModel):
    id: str

    created_at: datetime

    default: bool

    card: PaymentMethodCardData

    TYPE: Annotated[
        Annotated[Literal["card"], AfterValidator(validate_const("card"))],
        pydantic.Field(alias="type"),
    ] = "card"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subtype import SubType
from enum import Enum
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class TokenType(str, Enum):
    ACCESS_TOKEN = "access_token"
    REFRESH_TOKEN = "refresh_token"


class IntrospectTokenResponseTypedDict(TypedDict):
    active: bool
    client_id: str
    token_type: TokenType
    scope: str
    sub_type: SubType
    sub: str
    aud: str
    iss: str
    exp: int
    iat: int


class IntrospectTokenResponse(BaseModel):
    active: bool

    client_id: str

    token_type: TokenType

    scope: str

    sub_type: SubType

    sub: str

    aud: str

    iss: str

    exp: int

    iat: int



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ProductBillingType(str, Enum):
    ONE_TIME = "one_time"
    RECURRING = "recurring"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class Func(str, Enum):
    SUM = "sum"
    MAX = "max"
    MIN = "min"
    AVG = "avg"


class PropertyAggregationTypedDict(TypedDict):
    func: Func
    property: str


class PropertyAggregation(BaseModel):
    func: Func

    property: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .metricperiod import MetricPeriod, MetricPeriodTypedDict
from .metrics import Metrics, MetricsTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class MetricsResponseTypedDict(TypedDict):
    r"""Metrics response schema."""

    periods: List[MetricPeriodTypedDict]
    r"""List of data for each timestamp."""
    metrics: MetricsTypedDict


class MetricsResponse(BaseModel):
    r"""Metrics response schema."""

    periods: List[MetricPeriod]
    r"""List of data for each timestamp."""

    metrics: Metrics



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerstate import CustomerState, CustomerStateTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class WebhookCustomerStateChangedPayloadTypedDict(TypedDict):
    r"""Sent when a customer state has changed.

    It's triggered when:

    * Customer is created, updated or deleted.
    * A subscription is created or updated.
    * A benefit is granted or revoked.

    **Discord & Slack support:** Basic
    """

    data: CustomerStateTypedDict
    r"""A customer along with additional state information:

    * Active subscriptions
    * Active benefits
    """
    type: Literal["customer.state_changed"]


class WebhookCustomerStateChangedPayload(BaseModel):
    r"""Sent when a customer state has changed.

    It's triggered when:

    * Customer is created, updated or deleted.
    * A subscription is created or updated.
    * A benefit is granted or revoked.

    **Discord & Slack support:** Basic
    """

    data: CustomerState
    r"""A customer along with additional state information:

    * Active subscriptions
    * Active benefits
    """

    TYPE: Annotated[
        Annotated[
            Literal["customer.state_changed"],
            AfterValidator(validate_const("customer.state_changed")),
        ],
        pydantic.Field(alias="type"),
    ] = "customer.state_changed"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class BenefitGrantGitHubRepositoryPropertiesPermission(str, Enum):
    PULL = "pull"
    TRIAGE = "triage"
    PUSH = "push"
    MAINTAIN = "maintain"
    ADMIN = "admin"


class BenefitGrantGitHubRepositoryPropertiesTypedDict(TypedDict):
    account_id: NotRequired[str]
    repository_owner: NotRequired[str]
    repository_name: NotRequired[str]
    permission: NotRequired[BenefitGrantGitHubRepositoryPropertiesPermission]


class BenefitGrantGitHubRepositoryProperties(BaseModel):
    account_id: Optional[str] = None

    repository_owner: Optional[str] = None

    repository_name: Optional[str] = None

    permission: Optional[BenefitGrantGitHubRepositoryPropertiesPermission] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customersessioncustomerexternalidcreate import (
    CustomerSessionCustomerExternalIDCreate,
    CustomerSessionCustomerExternalIDCreateTypedDict,
)
from .customersessioncustomeridcreate import (
    CustomerSessionCustomerIDCreate,
    CustomerSessionCustomerIDCreateTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


CustomerSessionsCreateCustomerSessionCreateTypedDict = TypeAliasType(
    "CustomerSessionsCreateCustomerSessionCreateTypedDict",
    Union[
        CustomerSessionCustomerIDCreateTypedDict,
        CustomerSessionCustomerExternalIDCreateTypedDict,
    ],
)


CustomerSessionsCreateCustomerSessionCreate = TypeAliasType(
    "CustomerSessionsCreateCustomerSessionCreate",
    Union[CustomerSessionCustomerIDCreate, CustomerSessionCustomerExternalIDCreate],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customer import Customer, CustomerTypedDict
from .eventsource import EventSource
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Union
from typing_extensions import TypeAliasType, TypedDict


EventMetadataTypedDict = TypeAliasType("EventMetadataTypedDict", Union[str, int, bool])


EventMetadata = TypeAliasType("EventMetadata", Union[str, int, bool])


class EventTypedDict(TypedDict):
    metadata: Dict[str, EventMetadataTypedDict]
    id: str
    r"""The ID of the object."""
    timestamp: datetime
    r"""The timestamp of the event."""
    name: str
    r"""The name of the event."""
    source: EventSource
    organization_id: str
    r"""The ID of the organization owning the event."""
    customer_id: Nullable[str]
    r"""ID of the customer in your Polar organization associated with the event."""
    customer: Nullable[CustomerTypedDict]
    r"""The customer associated with the event."""
    external_customer_id: Nullable[str]
    r"""ID of the customer in your system associated with the event."""


class Event(BaseModel):
    metadata: Dict[str, EventMetadata]

    id: str
    r"""The ID of the object."""

    timestamp: datetime
    r"""The timestamp of the event."""

    name: str
    r"""The name of the event."""

    source: EventSource

    organization_id: str
    r"""The ID of the organization owning the event."""

    customer_id: Nullable[str]
    r"""ID of the customer in your Polar organization associated with the event."""

    customer: Nullable[Customer]
    r"""The customer associated with the event."""

    external_customer_id: Nullable[str]
    r"""ID of the customer in your system associated with the event."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["customer_id", "customer", "external_customer_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ProductPriceCustomCreateTypedDict(TypedDict):
    r"""Schema to create a pay-what-you-want price."""

    amount_type: Literal["custom"]
    price_currency: NotRequired[str]
    r"""The currency. Currently, only `usd` is supported."""
    minimum_amount: NotRequired[Nullable[int]]
    r"""The minimum amount the customer can pay."""
    maximum_amount: NotRequired[Nullable[int]]
    r"""The maximum amount the customer can pay."""
    preset_amount: NotRequired[Nullable[int]]
    r"""The initial amount shown to the customer."""


class ProductPriceCustomCreate(BaseModel):
    r"""Schema to create a pay-what-you-want price."""

    AMOUNT_TYPE: Annotated[
        Annotated[Literal["custom"], AfterValidator(validate_const("custom"))],
        pydantic.Field(alias="amount_type"),
    ] = "custom"

    price_currency: Optional[str] = "usd"
    r"""The currency. Currently, only `usd` is supported."""

    minimum_amount: OptionalNullable[int] = UNSET
    r"""The minimum amount the customer can pay."""

    maximum_amount: OptionalNullable[int] = UNSET
    r"""The maximum amount the customer can pay."""

    preset_amount: OptionalNullable[int] = UNSET
    r"""The initial amount shown to the customer."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "price_currency",
            "minimum_amount",
            "maximum_amount",
            "preset_amount",
        ]
        nullable_fields = ["minimum_amount", "maximum_amount", "preset_amount"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class BenefitType(str, Enum):
    CUSTOM = "custom"
    DISCORD = "discord"
    GITHUB_REPOSITORY = "github_repository"
    DOWNLOADABLES = "downloadables"
    LICENSE_KEYS = "license_keys"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkoutlinkcreateproduct import (
    CheckoutLinkCreateProduct,
    CheckoutLinkCreateProductTypedDict,
)
from .checkoutlinkcreateproductprice import (
    CheckoutLinkCreateProductPrice,
    CheckoutLinkCreateProductPriceTypedDict,
)
from .checkoutlinkcreateproducts import (
    CheckoutLinkCreateProducts,
    CheckoutLinkCreateProductsTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


CheckoutLinksCreateCheckoutLinkCreateTypedDict = TypeAliasType(
    "CheckoutLinksCreateCheckoutLinkCreateTypedDict",
    Union[
        CheckoutLinkCreateProductPriceTypedDict,
        CheckoutLinkCreateProductTypedDict,
        CheckoutLinkCreateProductsTypedDict,
    ],
)


CheckoutLinksCreateCheckoutLinkCreate = TypeAliasType(
    "CheckoutLinksCreateCheckoutLinkCreate",
    Union[
        CheckoutLinkCreateProductPrice,
        CheckoutLinkCreateProduct,
        CheckoutLinkCreateProducts,
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import date
from polar_sdk import models, utils
from polar_sdk._hooks import HookContext
from polar_sdk.types import OptionalNullable, UNSET
from typing import Any, Mapping, Optional, Union


class MetricsSDK(BaseSDK):
    def get(
        self,
        *,
        start_date: date,
        end_date: date,
        interval: models.TimeInterval,
        organization_id: OptionalNullable[
            Union[
                models.MetricsGetQueryParamOrganizationIDFilter,
                models.MetricsGetQueryParamOrganizationIDFilterTypedDict,
            ]
        ] = UNSET,
        product_id: OptionalNullable[
            Union[
                models.MetricsGetQueryParamProductIDFilter,
                models.MetricsGetQueryParamProductIDFilterTypedDict,
            ]
        ] = UNSET,
        billing_type: OptionalNullable[
            Union[
                models.QueryParamProductBillingTypeFilter,
                models.QueryParamProductBillingTypeFilterTypedDict,
            ]
        ] = UNSET,
        customer_id: OptionalNullable[
            Union[
                models.MetricsGetQueryParamCustomerIDFilter,
                models.MetricsGetQueryParamCustomerIDFilterTypedDict,
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.MetricsResponse:
        r"""Get Metrics

        Get metrics about your orders and subscriptions.

        Currency values are output in cents.

        **Scopes**: `metrics:read`

        :param start_date: Start date.
        :param end_date: End date.
        :param interval: Interval between two timestamps.
        :param organization_id: Filter by organization ID.
        :param product_id: Filter by product ID.
        :param billing_type: Filter by billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases.
        :param customer_id: Filter by customer ID.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.MetricsGetRequest(
            start_date=start_date,
            end_date=end_date,
            interval=interval,
            organization_id=organization_id,
            product_id=product_id,
            billing_type=billing_type,
            customer_id=customer_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/metrics/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="metrics:get",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.MetricsResponse)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        start_date: date,
        end_date: date,
        interval: models.TimeInterval,
        organization_id: OptionalNullable[
            Union[
                models.MetricsGetQueryParamOrganizationIDFilter,
                models.MetricsGetQueryParamOrganizationIDFilterTypedDict,
            ]
        ] = UNSET,
        product_id: OptionalNullable[
            Union[
                models.MetricsGetQueryParamProductIDFilter,
                models.MetricsGetQueryParamProductIDFilterTypedDict,
            ]
        ] = UNSET,
        billing_type: OptionalNullable[
            Union[
                models.QueryParamProductBillingTypeFilter,
                models.QueryParamProductBillingTypeFilterTypedDict,
            ]
        ] = UNSET,
        customer_id: OptionalNullable[
            Union[
                models.MetricsGetQueryParamCustomerIDFilter,
                models.MetricsGetQueryParamCustomerIDFilterTypedDict,
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.MetricsResponse:
        r"""Get Metrics

        Get metrics about your orders and subscriptions.

        Currency values are output in cents.

        **Scopes**: `metrics:read`

        :param start_date: Start date.
        :param end_date: End date.
        :param interval: Interval between two timestamps.
        :param organization_id: Filter by organization ID.
        :param product_id: Filter by product ID.
        :param billing_type: Filter by billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases.
        :param customer_id: Filter by customer ID.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.MetricsGetRequest(
            start_date=start_date,
            end_date=end_date,
            interval=interval,
            organization_id=organization_id,
            product_id=product_id,
            billing_type=billing_type,
            customer_id=customer_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/metrics/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="metrics:get",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.MetricsResponse)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def limits(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.MetricsLimits:
        r"""Get Metrics Limits

        Get the interval limits for the metrics endpoint.

        **Scopes**: `metrics:read`

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request(
            method="GET",
            path="/v1/metrics/limits",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="metrics:limits",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.MetricsLimits)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def limits_async(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.MetricsLimits:
        r"""Get Metrics Limits

        Get the interval limits for the metrics endpoint.

        **Scopes**: `metrics:read`

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request_async(
            method="GET",
            path="/v1/metrics/limits",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="metrics:limits",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.MetricsLimits)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .authorizationcodetokenrequest import (
    AuthorizationCodeTokenRequest,
    AuthorizationCodeTokenRequestTypedDict,
)
from .refreshtokenrequest import RefreshTokenRequest, RefreshTokenRequestTypedDict
from typing import Union
from typing_extensions import TypeAliasType


Oauth2RequestTokenRequestBodyTypedDict = TypeAliasType(
    "Oauth2RequestTokenRequestBodyTypedDict",
    Union[RefreshTokenRequestTypedDict, AuthorizationCodeTokenRequestTypedDict],
)


Oauth2RequestTokenRequestBody = TypeAliasType(
    "Oauth2RequestTokenRequestBody",
    Union[RefreshTokenRequest, AuthorizationCodeTokenRequest],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class TimeInterval(str, Enum):
    YEAR = "year"
    MONTH = "month"
    WEEK = "week"
    DAY = "day"
    HOUR = "hour"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerproduct import CustomerProduct, CustomerProductTypedDict
from .organization import Organization, OrganizationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class CustomerOrganizationTypedDict(TypedDict):
    r"""Schema of an organization and related data for customer portal."""

    organization: OrganizationTypedDict
    products: List[CustomerProductTypedDict]


class CustomerOrganization(BaseModel):
    r"""Schema of an organization and related data for customer portal."""

    organization: Organization

    products: List[CustomerProduct]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DiscountDuration(str, Enum):
    ONCE = "once"
    FOREVER = "forever"
    REPEATING = "repeating"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .attachedcustomfieldcreate import (
    AttachedCustomFieldCreate,
    AttachedCustomFieldCreateTypedDict,
)
from .productpricecustomcreate import (
    ProductPriceCustomCreate,
    ProductPriceCustomCreateTypedDict,
)
from .productpricefixedcreate import (
    ProductPriceFixedCreate,
    ProductPriceFixedCreateTypedDict,
)
from .productpricefreecreate import (
    ProductPriceFreeCreate,
    ProductPriceFreeCreateTypedDict,
)
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ProductCreateMetadataTypedDict = TypeAliasType(
    "ProductCreateMetadataTypedDict", Union[str, int, bool]
)


ProductCreateMetadata = TypeAliasType("ProductCreateMetadata", Union[str, int, bool])


ProductCreatePricesTypedDict = TypeAliasType(
    "ProductCreatePricesTypedDict",
    Union[
        ProductPriceFreeCreateTypedDict,
        ProductPriceFixedCreateTypedDict,
        ProductPriceCustomCreateTypedDict,
    ],
)


ProductCreatePrices = TypeAliasType(
    "ProductCreatePrices",
    Union[ProductPriceFreeCreate, ProductPriceFixedCreate, ProductPriceCustomCreate],
)


class ProductCreateTypedDict(TypedDict):
    r"""Schema to create a product."""

    name: str
    r"""The name of the product."""
    recurring_interval: Nullable[SubscriptionRecurringInterval]
    r"""The recurring interval of the product. If `None`, the product is a one-time purchase."""
    prices: List[ProductCreatePricesTypedDict]
    r"""List of available prices for this product. Currently, only a single price is supported."""
    metadata: NotRequired[Dict[str, ProductCreateMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    description: NotRequired[Nullable[str]]
    r"""The description of the product."""
    medias: NotRequired[Nullable[List[str]]]
    r"""List of file IDs. Each one must be on the same organization as the product, of type `product_media` and correctly uploaded."""
    attached_custom_fields: NotRequired[List[AttachedCustomFieldCreateTypedDict]]
    r"""List of custom fields to attach."""
    organization_id: NotRequired[Nullable[str]]
    r"""The ID of the organization owning the product. **Required unless you use an organization token.**"""


class ProductCreate(BaseModel):
    r"""Schema to create a product."""

    name: str
    r"""The name of the product."""

    recurring_interval: Nullable[SubscriptionRecurringInterval]
    r"""The recurring interval of the product. If `None`, the product is a one-time purchase."""

    prices: List[ProductCreatePrices]
    r"""List of available prices for this product. Currently, only a single price is supported."""

    metadata: Optional[Dict[str, ProductCreateMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    description: OptionalNullable[str] = UNSET
    r"""The description of the product."""

    medias: OptionalNullable[List[str]] = UNSET
    r"""List of file IDs. Each one must be on the same organization as the product, of type `product_media` and correctly uploaded."""

    attached_custom_fields: Optional[List[AttachedCustomFieldCreate]] = None
    r"""List of custom fields to attach."""

    organization_id: OptionalNullable[str] = UNSET
    r"""The ID of the organization owning the product. **Required unless you use an organization token.**"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "metadata",
            "description",
            "medias",
            "attached_custom_fields",
            "organization_id",
        ]
        nullable_fields = [
            "recurring_interval",
            "description",
            "medias",
            "organization_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .organizationsubscribepromotesettings import (
    OrganizationSubscribePromoteSettings,
    OrganizationSubscribePromoteSettingsTypedDict,
)
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List
from typing_extensions import NotRequired, TypedDict


class OrganizationProfileSettingsTypedDict(TypedDict):
    enabled: NotRequired[Nullable[bool]]
    r"""If this organization has a profile enabled"""
    description: NotRequired[Nullable[str]]
    r"""A description of the organization"""
    featured_projects: NotRequired[Nullable[List[str]]]
    r"""A list of featured projects"""
    featured_organizations: NotRequired[Nullable[List[str]]]
    r"""A list of featured organizations"""
    links: NotRequired[Nullable[List[str]]]
    r"""A list of links associated with the organization"""
    subscribe: NotRequired[Nullable[OrganizationSubscribePromoteSettingsTypedDict]]
    r"""Subscription promotion settings"""
    accent_color: NotRequired[Nullable[str]]
    r"""Accent color for the organization"""


class OrganizationProfileSettings(BaseModel):
    enabled: OptionalNullable[bool] = UNSET
    r"""If this organization has a profile enabled"""

    description: OptionalNullable[str] = UNSET
    r"""A description of the organization"""

    featured_projects: OptionalNullable[List[str]] = UNSET
    r"""A list of featured projects"""

    featured_organizations: OptionalNullable[List[str]] = UNSET
    r"""A list of featured organizations"""

    links: OptionalNullable[List[str]] = UNSET
    r"""A list of links associated with the organization"""

    subscribe: OptionalNullable[OrganizationSubscribePromoteSettings] = UNSET
    r"""Subscription promotion settings"""

    accent_color: OptionalNullable[str] = UNSET
    r"""Accent color for the organization"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "enabled",
            "description",
            "featured_projects",
            "featured_organizations",
            "links",
            "subscribe",
            "accent_color",
        ]
        nullable_fields = [
            "enabled",
            "description",
            "featured_projects",
            "featured_organizations",
            "links",
            "subscribe",
            "accent_color",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, SecurityMetadata
from typing_extensions import Annotated, TypedDict


class CustomerPortalCustomersUpdateSecurityTypedDict(TypedDict):
    customer_session: str


class CustomerPortalCustomersUpdateSecurity(BaseModel):
    customer_session: Annotated[
        str,
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .countaggregation import CountAggregation, CountAggregationTypedDict
from .filter_ import Filter, FilterTypedDict
from .propertyaggregation import PropertyAggregation, PropertyAggregationTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from polar_sdk.utils import get_discriminator
import pydantic
from pydantic import Discriminator, Tag, model_serializer
from typing import Dict, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


MeterMetadataTypedDict = TypeAliasType("MeterMetadataTypedDict", Union[str, int, bool])


MeterMetadata = TypeAliasType("MeterMetadata", Union[str, int, bool])


MeterAggregationTypedDict = TypeAliasType(
    "MeterAggregationTypedDict",
    Union[CountAggregationTypedDict, PropertyAggregationTypedDict],
)
r"""The aggregation to apply on the filtered events to calculate the meter."""


MeterAggregation = Annotated[
    Union[
        Annotated[PropertyAggregation, Tag("avg")],
        Annotated[CountAggregation, Tag("count")],
        Annotated[PropertyAggregation, Tag("max")],
        Annotated[PropertyAggregation, Tag("min")],
        Annotated[PropertyAggregation, Tag("sum")],
    ],
    Discriminator(lambda m: get_discriminator(m, "func", "func")),
]
r"""The aggregation to apply on the filtered events to calculate the meter."""


class MeterTypedDict(TypedDict):
    metadata: Dict[str, MeterMetadataTypedDict]
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    name: str
    r"""The name of the meter. Will be shown on customer's invoices and usage."""
    filter_: FilterTypedDict
    aggregation: MeterAggregationTypedDict
    r"""The aggregation to apply on the filtered events to calculate the meter."""
    organization_id: str
    r"""The ID of the organization owning the meter."""


class Meter(BaseModel):
    metadata: Dict[str, MeterMetadata]

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    name: str
    r"""The name of the meter. Will be shown on customer's invoices and usage."""

    filter_: Annotated[Filter, pydantic.Field(alias="filter")]

    aggregation: MeterAggregation
    r"""The aggregation to apply on the filtered events to calculate the meter."""

    organization_id: str
    r"""The ID of the organization owning the meter."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class ProductPriceFreeCreateTypedDict(TypedDict):
    r"""Schema to create a free price."""

    amount_type: Literal["free"]


class ProductPriceFreeCreate(BaseModel):
    r"""Schema to create a free price."""

    AMOUNT_TYPE: Annotated[
        Annotated[Literal["free"], AfterValidator(validate_const("free"))],
        pydantic.Field(alias="amount_type"),
    ] = "free"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CustomerCancellationReason(str, Enum):
    CUSTOMER_SERVICE = "customer_service"
    LOW_QUALITY = "low_quality"
    MISSING_FEATURES = "missing_features"
    SWITCHED_SERVICE = "switched_service"
    TOO_COMPLEX = "too_complex"
    TOO_EXPENSIVE = "too_expensive"
    UNUSED = "unused"
    OTHER = "other"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CustomerSubscriptionSortProperty(str, Enum):
    STARTED_AT = "started_at"
    MINUS_STARTED_AT = "-started_at"
    AMOUNT = "amount"
    MINUS_AMOUNT = "-amount"
    STATUS = "status"
    MINUS_STATUS = "-status"
    ORGANIZATION = "organization"
    MINUS_ORGANIZATION = "-organization"
    PRODUCT = "product"
    MINUS_PRODUCT = "-product"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomerOrderInvoiceTypedDict(TypedDict):
    r"""Order's invoice data."""

    url: str
    r"""The URL to the invoice."""


class CustomerOrderInvoice(BaseModel):
    r"""Order's invoice data."""

    url: str
    r"""The URL to the invoice."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CheckoutLinkSortProperty(str, Enum):
    CREATED_AT = "created_at"
    MINUS_CREATED_AT = "-created_at"
    LABEL = "label"
    MINUS_LABEL = "-label"
    SUCCESS_URL = "success_url"
    MINUS_SUCCESS_URL = "-success_url"
    ALLOW_DISCOUNT_CODES = "allow_discount_codes"
    MINUS_ALLOW_DISCOUNT_CODES = "-allow_discount_codes"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SubType(str, Enum):
    USER = "user"
    ORGANIZATION = "organization"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class MeterQuantityTypedDict(TypedDict):
    timestamp: datetime
    r"""The timestamp for the current period."""
    quantity: float
    r"""The quantity for the current period."""


class MeterQuantity(BaseModel):
    timestamp: datetime
    r"""The timestamp for the current period."""

    quantity: float
    r"""The quantity for the current period."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkoutupdatepublic import CheckoutUpdatePublic, CheckoutUpdatePublicTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class CheckoutsClientUpdateRequestTypedDict(TypedDict):
    client_secret: str
    r"""The checkout session client secret."""
    checkout_update_public: CheckoutUpdatePublicTypedDict


class CheckoutsClientUpdateRequest(BaseModel):
    client_secret: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The checkout session client secret."""

    checkout_update_public: Annotated[
        CheckoutUpdatePublic,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .listresource_event_ import ListResourceEvent, ListResourceEventTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing import Callable, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class MetersEventsRequestTypedDict(TypedDict):
    id: str
    r"""The meter ID."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""


class MetersEventsRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The meter ID."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""


class MetersEventsResponseTypedDict(TypedDict):
    result: ListResourceEventTypedDict


class MetersEventsResponse(BaseModel):
    next: Callable[[], Optional[MetersEventsResponse]]

    result: ListResourceEvent



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomerPaymentMethodCreateTypedDict(TypedDict):
    confirmation_token_id: str
    set_default: bool
    return_url: str


class CustomerPaymentMethodCreate(BaseModel):
    confirmation_token_id: str

    set_default: bool

    return_url: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
import httpx
from polar_sdk import models, utils
from polar_sdk._hooks import SDKHooks
from polar_sdk.benefits import Benefits
from polar_sdk.checkout_links import CheckoutLinks
from polar_sdk.checkouts import Checkouts
from polar_sdk.custom_fields import CustomFields
from polar_sdk.customer_portal import CustomerPortal
from polar_sdk.customer_sessions import CustomerSessions
from polar_sdk.customers import Customers
from polar_sdk.discounts import Discounts
from polar_sdk.events import Events
from polar_sdk.external_organizations import ExternalOrganizations
from polar_sdk.files import Files
from polar_sdk.license_keys import LicenseKeys
from polar_sdk.meters import Meters
from polar_sdk.metrics_sdk import MetricsSDK
from polar_sdk.oauth2 import Oauth2
from polar_sdk.orders import Orders
from polar_sdk.organizations import Organizations
from polar_sdk.products import Products
from polar_sdk.refunds import Refunds
from polar_sdk.repositories import Repositories
from polar_sdk.subscriptions import Subscriptions
from polar_sdk.types import OptionalNullable, UNSET
from typing import Any, Callable, Dict, Optional, Union, cast
import weakref


class Polar(BaseSDK):
    r"""Polar API: Polar HTTP and Webhooks API

    Read the docs at https://docs.polar.sh/api-reference
    """

    external_organizations: ExternalOrganizations
    repositories: Repositories
    organizations: Organizations
    subscriptions: Subscriptions
    oauth2: Oauth2
    benefits: Benefits
    products: Products
    orders: Orders
    refunds: Refunds
    checkouts: Checkouts
    files: Files
    metrics: MetricsSDK
    license_keys: LicenseKeys
    checkout_links: CheckoutLinks
    custom_fields: CustomFields
    discounts: Discounts
    customers: Customers
    customer_portal: CustomerPortal
    customer_sessions: CustomerSessions
    events: Events
    meters: Meters

    def __init__(
        self,
        access_token: Optional[
            Union[Optional[str], Callable[[], Optional[str]]]
        ] = None,
        server: Optional[str] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param access_token: The access_token required for authentication
        :param server: The server by name to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client()
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient()
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(access_token):
            # pylint: disable=unnecessary-lambda-assignment
            security = lambda: models.Security(access_token=access_token())
        else:
            security = models.Security(access_token=access_token)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                security=security,
                server_url=server_url,
                server=server,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
        )

        hooks = SDKHooks()

        current_server_url, *_ = self.sdk_configuration.get_server_details()
        server_url, self.sdk_configuration.client = hooks.sdk_init(
            current_server_url, client
        )
        if current_server_url != server_url:
            self.sdk_configuration.server_url = server_url

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

        self._init_sdks()

    def _init_sdks(self):
        self.external_organizations = ExternalOrganizations(self.sdk_configuration)
        self.repositories = Repositories(self.sdk_configuration)
        self.organizations = Organizations(self.sdk_configuration)
        self.subscriptions = Subscriptions(self.sdk_configuration)
        self.oauth2 = Oauth2(self.sdk_configuration)
        self.benefits = Benefits(self.sdk_configuration)
        self.products = Products(self.sdk_configuration)
        self.orders = Orders(self.sdk_configuration)
        self.refunds = Refunds(self.sdk_configuration)
        self.checkouts = Checkouts(self.sdk_configuration)
        self.files = Files(self.sdk_configuration)
        self.metrics = MetricsSDK(self.sdk_configuration)
        self.license_keys = LicenseKeys(self.sdk_configuration)
        self.checkout_links = CheckoutLinks(self.sdk_configuration)
        self.custom_fields = CustomFields(self.sdk_configuration)
        self.discounts = Discounts(self.sdk_configuration)
        self.customers = Customers(self.sdk_configuration)
        self.customer_portal = CustomerPortal(self.sdk_configuration)
        self.customer_sessions = CustomerSessions(self.sdk_configuration)
        self.events = Events(self.sdk_configuration)
        self.meters = Meters(self.sdk_configuration)

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitcustomcreate import BenefitCustomCreate, BenefitCustomCreateTypedDict
from .benefitdiscordcreate import BenefitDiscordCreate, BenefitDiscordCreateTypedDict
from .benefitdownloadablescreate import (
    BenefitDownloadablesCreate,
    BenefitDownloadablesCreateTypedDict,
)
from .benefitgithubrepositorycreate import (
    BenefitGitHubRepositoryCreate,
    BenefitGitHubRepositoryCreateTypedDict,
)
from .benefitlicensekeyscreate import (
    BenefitLicenseKeysCreate,
    BenefitLicenseKeysCreateTypedDict,
)
from polar_sdk.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


BenefitCreateTypedDict = TypeAliasType(
    "BenefitCreateTypedDict",
    Union[
        BenefitCustomCreateTypedDict,
        BenefitDiscordCreateTypedDict,
        BenefitGitHubRepositoryCreateTypedDict,
        BenefitDownloadablesCreateTypedDict,
        BenefitLicenseKeysCreateTypedDict,
    ],
)


BenefitCreate = Annotated[
    Union[
        Annotated[BenefitCustomCreate, Tag("custom")],
        Annotated[BenefitDiscordCreate, Tag("discord")],
        Annotated[BenefitDownloadablesCreate, Tag("downloadables")],
        Annotated[BenefitGitHubRepositoryCreate, Tag("github_repository")],
        Annotated[BenefitLicenseKeysCreate, Tag("license_keys")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .organizationfeaturesettings import (
    OrganizationFeatureSettings,
    OrganizationFeatureSettingsTypedDict,
)
from .organizationprofilesettings import (
    OrganizationProfileSettings,
    OrganizationProfileSettingsTypedDict,
)
from .organizationsociallink import (
    OrganizationSocialLink,
    OrganizationSocialLinkTypedDict,
)
from .organizationsubscriptionsettings import (
    OrganizationSubscriptionSettings,
    OrganizationSubscriptionSettingsTypedDict,
)
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List
from typing_extensions import Annotated, TypedDict


class OrganizationTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The organization ID."""
    name: str
    r"""Organization name shown in checkout, customer portal, emails etc."""
    slug: str
    r"""Unique organization slug in checkout, customer portal and credit card statements."""
    avatar_url: Nullable[str]
    r"""Avatar URL shown in checkout, customer portal, emails etc."""
    email: Nullable[str]
    r"""Public support email."""
    website: Nullable[str]
    r"""Official website of the organization."""
    socials: List[OrganizationSocialLinkTypedDict]
    r"""Links to social profiles."""
    details_submitted_at: Nullable[datetime]
    r"""When the business details were submitted."""
    feature_settings: Nullable[OrganizationFeatureSettingsTypedDict]
    r"""Organization feature settings"""
    subscription_settings: OrganizationSubscriptionSettingsTypedDict
    bio: Nullable[str]
    company: Nullable[str]
    blog: Nullable[str]
    location: Nullable[str]
    twitter_username: Nullable[str]
    pledge_minimum_amount: int
    pledge_badge_show_amount: bool
    default_upfront_split_to_contributors: Nullable[int]
    profile_settings: Nullable[OrganizationProfileSettingsTypedDict]
    r"""Settings for the organization profile"""


class Organization(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The organization ID."""

    name: str
    r"""Organization name shown in checkout, customer portal, emails etc."""

    slug: str
    r"""Unique organization slug in checkout, customer portal and credit card statements."""

    avatar_url: Nullable[str]
    r"""Avatar URL shown in checkout, customer portal, emails etc."""

    email: Nullable[str]
    r"""Public support email."""

    website: Nullable[str]
    r"""Official website of the organization."""

    socials: List[OrganizationSocialLink]
    r"""Links to social profiles."""

    details_submitted_at: Nullable[datetime]
    r"""When the business details were submitted."""

    feature_settings: Nullable[OrganizationFeatureSettings]
    r"""Organization feature settings"""

    subscription_settings: OrganizationSubscriptionSettings

    bio: Annotated[
        Nullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]

    company: Annotated[
        Nullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]

    blog: Annotated[
        Nullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]

    location: Annotated[
        Nullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]

    twitter_username: Annotated[
        Nullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]

    pledge_minimum_amount: Annotated[
        int,
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]

    pledge_badge_show_amount: Annotated[
        bool,
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]

    default_upfront_split_to_contributors: Annotated[
        Nullable[int],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]

    profile_settings: Annotated[
        Nullable[OrganizationProfileSettings],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]
    r"""Settings for the organization profile"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "modified_at",
            "avatar_url",
            "email",
            "website",
            "details_submitted_at",
            "feature_settings",
            "bio",
            "company",
            "blog",
            "location",
            "twitter_username",
            "default_upfront_split_to_contributors",
            "profile_settings",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .refundreason import RefundReason
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


RefundCreateMetadataTypedDict = TypeAliasType(
    "RefundCreateMetadataTypedDict", Union[str, int, bool]
)


RefundCreateMetadata = TypeAliasType("RefundCreateMetadata", Union[str, int, bool])


class RefundCreateTypedDict(TypedDict):
    order_id: str
    reason: RefundReason
    amount: int
    r"""Amount to refund in cents. Minimum is 1."""
    metadata: NotRequired[Dict[str, RefundCreateMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    comment: NotRequired[Nullable[str]]
    r"""An internal comment about the refund."""
    revoke_benefits: NotRequired[bool]
    r"""Should this refund trigger the associated customer benefits to be revoked?

    **Note:**
    Only allowed in case the `order` is a one-time purchase.
    Subscriptions automatically revoke customer benefits once the
    subscription itself is revoked, i.e fully canceled.
    """


class RefundCreate(BaseModel):
    order_id: str

    reason: RefundReason

    amount: int
    r"""Amount to refund in cents. Minimum is 1."""

    metadata: Optional[Dict[str, RefundCreateMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    comment: OptionalNullable[str] = UNSET
    r"""An internal comment about the refund."""

    revoke_benefits: Optional[bool] = False
    r"""Should this refund trigger the associated customer benefits to be revoked?

    **Note:**
    Only allowed in case the `order` is a one-time purchase.
    Subscriptions automatically revoke customer benefits once the
    subscription itself is revoked, i.e fully canceled.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["metadata", "comment", "revoke_benefits"]
        nullable_fields = ["comment"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class FilterOperator(str, Enum):
    EQ = "eq"
    NE = "ne"
    GT = "gt"
    GTE = "gte"
    LT = "lt"
    LTE = "lte"
    LIKE = "like"
    NOT_LIKE = "not_like"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from jsonpath import JSONPath
from polar_sdk import models, utils
from polar_sdk._hooks import HookContext
from polar_sdk.types import BaseModel, OptionalNullable, UNSET
from typing import Any, Dict, Mapping, Optional, Union, cast


class Clients(BaseSDK):
    def list(
        self,
        *,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.Oauth2ClientsListResponse]:
        r"""List Clients

        List OAuth2 clients.

        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.Oauth2ClientsListRequest(
            page=page,
            limit=limit,
        )

        req = self._build_request(
            method="GET",
            path="/v1/oauth2/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="oauth2:clients:list",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.Oauth2ClientsListResponse]:
            body = utils.unmarshal_json(http_res.text, Dict[Any, Any])
            page = request.page if not request.page is None else 1
            next_page = page + 1

            num_pages = JSONPath("$.pagination.max_page").parse(body)
            if len(num_pages) == 0 or num_pages[0] <= page:
                return None

            if not http_res.text:
                return None
            results = JSONPath("$.items").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.limit if not request.limit is None else 10
            if len(results[0]) < limit:
                return None

            return self.list(
                page=next_page,
                limit=limit,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.Oauth2ClientsListResponse(
                result=utils.unmarshal_json(
                    http_res.text, models.ListResourceOAuth2Client
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def list_async(
        self,
        *,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.Oauth2ClientsListResponse]:
        r"""List Clients

        List OAuth2 clients.

        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.Oauth2ClientsListRequest(
            page=page,
            limit=limit,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/oauth2/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="oauth2:clients:list",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.Oauth2ClientsListResponse]:
            body = utils.unmarshal_json(http_res.text, Dict[Any, Any])
            page = request.page if not request.page is None else 1
            next_page = page + 1

            num_pages = JSONPath("$.pagination.max_page").parse(body)
            if len(num_pages) == 0 or num_pages[0] <= page:
                return None

            if not http_res.text:
                return None
            results = JSONPath("$.items").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.limit if not request.limit is None else 10
            if len(results[0]) < limit:
                return None

            return self.list(
                page=next_page,
                limit=limit,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.Oauth2ClientsListResponse(
                result=utils.unmarshal_json(
                    http_res.text, models.ListResourceOAuth2Client
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def create(
        self,
        *,
        request: Union[
            models.OAuth2ClientConfiguration, models.OAuth2ClientConfigurationTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Create Client

        Create an OAuth2 client.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.OAuth2ClientConfiguration)
        request = cast(models.OAuth2ClientConfiguration, request)

        req = self._build_request(
            method="POST",
            path="/v1/oauth2/register",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.OAuth2ClientConfiguration
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="oauth2:clients:oauth2:create_client",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def create_async(
        self,
        *,
        request: Union[
            models.OAuth2ClientConfiguration, models.OAuth2ClientConfigurationTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Create Client

        Create an OAuth2 client.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.OAuth2ClientConfiguration)
        request = cast(models.OAuth2ClientConfiguration, request)

        req = self._build_request_async(
            method="POST",
            path="/v1/oauth2/register",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.OAuth2ClientConfiguration
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="oauth2:clients:oauth2:create_client",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get(
        self,
        *,
        client_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Get Client

        Get an OAuth2 client by Client ID.

        :param client_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.Oauth2ClientsOauth2GetClientRequest(
            client_id=client_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/oauth2/register/{client_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="oauth2:clients:oauth2:get_client",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        client_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Get Client

        Get an OAuth2 client by Client ID.

        :param client_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.Oauth2ClientsOauth2GetClientRequest(
            client_id=client_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/oauth2/register/{client_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="oauth2:clients:oauth2:get_client",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def update(
        self,
        *,
        client_id: str,
        o_auth2_client_configuration_update: Union[
            models.OAuth2ClientConfigurationUpdate,
            models.OAuth2ClientConfigurationUpdateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Update Client

        Update an OAuth2 client.

        :param client_id:
        :param o_auth2_client_configuration_update:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.Oauth2ClientsOauth2UpdateClientRequest(
            client_id=client_id,
            o_auth2_client_configuration_update=utils.get_pydantic_model(
                o_auth2_client_configuration_update,
                models.OAuth2ClientConfigurationUpdate,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v1/oauth2/register/{client_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.o_auth2_client_configuration_update,
                False,
                False,
                "json",
                models.OAuth2ClientConfigurationUpdate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="oauth2:clients:oauth2:update_client",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def update_async(
        self,
        *,
        client_id: str,
        o_auth2_client_configuration_update: Union[
            models.OAuth2ClientConfigurationUpdate,
            models.OAuth2ClientConfigurationUpdateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Update Client

        Update an OAuth2 client.

        :param client_id:
        :param o_auth2_client_configuration_update:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.Oauth2ClientsOauth2UpdateClientRequest(
            client_id=client_id,
            o_auth2_client_configuration_update=utils.get_pydantic_model(
                o_auth2_client_configuration_update,
                models.OAuth2ClientConfigurationUpdate,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v1/oauth2/register/{client_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.o_auth2_client_configuration_update,
                False,
                False,
                "json",
                models.OAuth2ClientConfigurationUpdate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="oauth2:clients:oauth2:update_client",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def delete(
        self,
        *,
        client_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Delete Client

        Delete an OAuth2 client.

        :param client_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.Oauth2ClientsOauth2DeleteClientRequest(
            client_id=client_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/v1/oauth2/register/{client_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="oauth2:clients:oauth2:delete_client",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def delete_async(
        self,
        *,
        client_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Delete Client

        Delete an OAuth2 client.

        :param client_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.Oauth2ClientsOauth2DeleteClientRequest(
            client_id=client_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v1/oauth2/register/{client_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="oauth2:clients:oauth2:delete_client",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from polar_sdk.benefit_grants import BenefitGrants
from polar_sdk.downloadables import Downloadables
from polar_sdk.polar_customers import PolarCustomers
from polar_sdk.polar_license_keys import PolarLicenseKeys
from polar_sdk.polar_orders import PolarOrders
from polar_sdk.polar_organizations import PolarOrganizations
from polar_sdk.polar_subscriptions import PolarSubscriptions


class CustomerPortal(BaseSDK):
    benefit_grants: BenefitGrants
    customers: PolarCustomers
    downloadables: Downloadables
    license_keys: PolarLicenseKeys
    orders: PolarOrders
    organizations: PolarOrganizations
    subscriptions: PolarSubscriptions

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.benefit_grants = BenefitGrants(self.sdk_configuration)
        self.customers = PolarCustomers(self.sdk_configuration)
        self.downloadables = Downloadables(self.sdk_configuration)
        self.license_keys = PolarLicenseKeys(self.sdk_configuration)
        self.orders = PolarOrders(self.sdk_configuration)
        self.organizations = PolarOrganizations(self.sdk_configuration)
        self.subscriptions = PolarSubscriptions(self.sdk_configuration)
